@@ -20,6 +20,7 @@ import Data.Set (Set)
 import qualified Data.Set as Set
 import Data.Map (Map)
 import qualified Data.Map as Map
+import qualified Data.Map.Strict as MapStrict
 import Data.IntMap (IntMap)
 import qualified Data.IntMap as IntMap
 import Data.IntSet (IntSet)
@@ -161,11 +162,11 @@ kennedyWarrenOrder opts wr ndis typesyns derivings = runST $ runErrorT $ do
 -- | Pretty print a vertex in GraphViz format
 toGVVertex :: Bool -> Vertex -> ST s PP_Doc
 toGVVertex l (VAttr t a b) = return $ (text $ "attr_" ++ show t ++ "_" ++ show a ++ "_" ++ show b) >#< if l
-	     	      	   	then text ("[shape=box,label=\"" ++ show t ++ " @" ++ show a ++ "." ++ show b ++ "\"]") else empty
+                        then text ("[shape=box,label=\"" ++ show t ++ " @" ++ show a ++ "." ++ show b ++ "\"]") else empty
 toGVVertex l (VChild c)    = return $ (text $ "child_" ++ show c) >#< if l
-	     	      	   	then text ("[shape=ellipse,label=\"Child " ++ show c ++ "\"]") else empty
+                        then text ("[shape=ellipse,label=\"Child " ++ show c ++ "\"]") else empty
 toGVVertex l (VRule r)   = return $ (text $ "rule_"  ++ show r) >#< if l
-	     	      	   	then text ("[shape=diamond,label=\"" ++ show r ++ "\"]") else empty
+                        then text ("[shape=diamond,label=\"" ++ show r ++ "\"]") else empty
 
 -- | Pretty print an edge in GraphViz format
 toGVEdge :: Edge -> ST s PP_Doc
@@ -400,7 +401,7 @@ addChildVisit (VGProd (VGEdge edg, p)) ide (VGEdge vs) = do
   let (VGNode from,vgto) = imLookup vs edges -- from must be equal to the current state
   childvs <- gets vgChildVisits
   let rchildv = imLookup edg childvs
-  vgInST $ modifySTRef rchildv $ Map.insertWith' (++) (ide,p) [vgto]
+  vgInST $ modifySTRef rchildv $ MapStrict.insertWith (++) (ide,p) [vgto]
   ndis <- gets vgNDI
   let rndi = imLookup from ndis
   ndi <- vgInST $ readSTRef rndi
@@ -766,10 +767,10 @@ kennedyWarrenExecutionPlan opts ndis initvs wr typesyns derivings = do
       -- Return execution plan for this production
       return $ EProduction (pdgProduction $ pdgmOrig prod)
                            (pdgParams     $ pdgmOrig prod)
-			   (pdgConstraints $ pdgmOrig prod)
+               (pdgConstraints $ pdgmOrig prod)
                            (pdgRules      $ pdgmOrig prod)
                            (pdgChilds     $ pdgmOrig prod)
-			   visits
+               visits
     -- Find initial state for this nonterminal
     VGNode init <- vgFindInitial $ ndiNonterminal $ ndimOrig ndi
     -- Construct an environment that specifies the next visit of the states that have exactly one
